@@ -24,7 +24,7 @@ import re
 import glob
 from userprefs import Options, UserPrefs
 from cache import WCache
-from vars import Vars
+from vars import Vars, getuser
 from utils import *
 from msg import *
 
@@ -168,6 +168,32 @@ def ra_types(ra_class = "ocf", ra_provider = ""):
     list.sort()
     return wcache.store(id,list)
 
+def get_pe_meta():
+    if not vars.pe_metadata:
+        vars.pe_metadata = RAInfo("pengine","metadata")
+    return vars.pe_metadata
+def get_crmd_meta():
+    if not vars.crmd_metadata:
+        vars.crmd_metadata = RAInfo("crmd","metadata")
+        vars.crmd_metadata.set_advanced_params(vars.crmd_advanced)
+    return vars.crmd_metadata
+def get_stonithd_meta():
+    if not vars.stonithd_metadata:
+        vars.stonithd_metadata = RAInfo("stonithd","metadata")
+    return vars.stonithd_metadata
+def get_properties_meta():
+    if not vars.crm_properties_metadata:
+        get_pe_meta()
+        get_crmd_meta()
+        vars.crm_properties_metadata = copy.deepcopy(vars.crmd_metadata)
+        vars.crm_properties_metadata.add_ra_params(vars.pe_metadata)
+    return vars.crm_properties_metadata
+def get_properties_list():
+    try:
+        return get_properties_meta().params().keys()
+    except:
+        return []
+
 def prog_meta(s):
     '''
     Do external program metadata.
@@ -186,7 +212,7 @@ def get_nodes_text(n,tag):
     except: return ''
 
 def mk_monitor_name(role,depth):
-    depth = depth == "0" and "" or ("_%s" % depth)
+    depth = depth != "0" and ("_%s" % depth) or ""
     return role and role != "Started" and \
         "monitor_%s%s" % (role,depth) or \
         "monitor%s" % depth
@@ -241,6 +267,7 @@ class RAInfo(object):
     skip_ops = ("meta-data", "validate-all")
     skip_op_attr = ("name", "depth", "role")
     def __init__(self,ra_class,ra_type,ra_provider = "heartbeat"):
+        self.advanced_params = []
         self.ra_class = ra_class
         self.ra_type = ra_type
         self.ra_provider = ra_provider
@@ -255,17 +282,23 @@ class RAInfo(object):
         common_err("%s: %s" % (self.ra_string(), s))
     def warn(self, s):
         common_warn("%s: %s" % (self.ra_string(), s))
-    def add_extra_stonith_params(self):
-        if not vars.stonithd_metadata:
-            vars.stonithd_metadata = RAInfo("stonithd","metadata")
-        if not vars.stonithd_metadata.mk_ra_node():
+    def set_advanced_params(self, l):
+        self.advanced_params = l
+    def add_ra_params(self,ra):
+        '''
+        Add parameters from another RAInfo instance.
+        '''
+        try:
+            if not self.mk_ra_node() or not ra.mk_ra_node():
+                return
+        except:
             return
         try:
             params_node = self.doc.getElementsByTagName("parameters")[0]
         except:
             params_node = self.doc.createElement("parameters")
             self.ra_node.appendChild(params_node)
-        for n in vars.stonithd_metadata.ra_node.getElementsByTagName("parameter"):
+        for n in ra.ra_node.getElementsByTagName("parameter"):
             params_node.appendChild(self.doc.importNode(n,1))
     def mk_ra_node(self):
         '''
@@ -287,7 +320,7 @@ class RAInfo(object):
             self.ra_node = None
             return None
         if self.ra_class == "stonith":
-            self.add_extra_stonith_params()
+            self.add_ra_params(get_stonithd_meta())
         return self.ra_node
     def param_type_default(self,n):
         try:
@@ -323,6 +356,18 @@ class RAInfo(object):
                     "default": default,
                 }
         return wcache.store(id,d)
+    def completion_params(self):
+        '''
+        Extra method for completion, for we want to filter some
+        (advanced) parameters out. And we want this to be fast.
+        '''
+        if not self.mk_ra_node():
+            return None
+        return [c.getAttribute("name")
+            for c in self.ra_node.getElementsByTagName("parameter")
+                if c.getAttribute("name")
+                and c.getAttribute("name") not in self.advanced_params
+        ]
     def actions(self):
         '''
         Construct a dict of dicts: actions are keys and
@@ -424,6 +469,11 @@ class RAInfo(object):
                 common_warn("%s: action %s not advertised in meta-data, it may not be supported by the RA" % (id,op))
                 rc |= 1
                 continue
+	    if "interval" in n_ops[op]:
+                if op == "start" or op == "stop":
+                    v = n_ops[op]["interval"]
+                    if crm_msec(v) != 0:
+                        common_warn("%s: Specified interval for %s is %s, it must be 0" %(id,op,v))
             try:
                 adv_timeout = self.actions()[op]["timeout"]
             except:
@@ -583,7 +633,7 @@ def disambiguate_ra_type(s):
     '''
     l = s.split(':')
     if not l or len(l) > 3:
-        return None
+        return ["","",""]
     if len(l) == 3:
         return l
     elif len(l) == 2:
